/*
  media-filesystem object export code
  tridge@samba.org, September 2002
  released under the Gnu GPL v2
*/

#include "mfs.h"

void export_file(FILE *log, u32 fsid, char *dest, u64 start, u64 count)
{
	int fd;
	void *buf;
	int bufsize = 128*1024;
	int n;
	u64 ofs;
	u64 size, total=0;
	int pct, last_pct;

	ofs = start;

	fd = open(dest, O_WRONLY|O_CREAT|O_TRUNC|O_LARGEFILE, 0644);
	if (fd == -1) {
		perror(dest);
		exit(1);
	}
	buf = malloc(bufsize);
	mfs_readahead(1);
	size = mfs_fsid_size(fsid);
	fprintf(log, "exporting fsid %d of size %lld to %s\n", fsid, size, dest);
	if (start > size) {
		printf("start beyond EOF!\n");
		exit(1);
	}
	if (start+count>size || count==0) {
		count = size-start;
	}
	if (start || count) {
		fprintf(log, "starting at %lld for %lld bytes\n", start, count);
	}

	last_pct=0;

	while (total<count &&
	       (n=mfs_fsid_pread(fsid, buf, ofs, MIN(bufsize,count-total))) > 0) {
		if (write(fd, buf, n) != n) {
			fprintf(stderr,"failed to write to %s\n", dest);
			break;
		}
		ofs += n;
		total += n;
		pct = (100*total)/count;
		if (pct != last_pct) {
			fprintf(log, "%d%%\r", pct);
			fflush(log);
			last_pct = pct;
		}
	}
	close(fd);
}
