/*
  media-filesystem object dump
  tridge@samba.org, January 2001
  released under the Gnu GPL v2
*/

#include "mfs.h"

static void dump_callback(int fsid, struct mfs_subobj_header *obj,
			  struct mfs_attr_header *attr, void *data)
{
	int i;
	char *p = data;
	static int last;
	struct mfs_obj_attr *objattr;

	if (!attr) {
		if (last) printf("}\n");
		last = fsid;
		printf("%s %d/%d %s{\n",
		       schema_type(obj->obj_type), fsid, obj->id,
		       obj->flags?"PRIMARY ":"");
		return;
	}

	if (schema_attrib(obj->obj_type,attr->attr)) {
		printf("\t%s[%d]=", 
		    schema_attrib(obj->obj_type,attr->attr), attr->attr);
	} else {
		printf("\t[%d]=", attr->attr);
	}
        switch (attr->eltype>>6) {
        case TYPE_STRING:
                for (i=0;i<attr->len-4;) {
                        char *s = (char *)&p[i];
                        printf("%s ", s);
                        i += strlen(s)+1;
                }
                break;
        case TYPE_INT:
        case TYPE_FILE:
		for (i=0;i<(attr->len-4)/4;i++) {
		    printf("%d ",ntohl(*(int *)&p[i*4]));
		}
                break;
        case TYPE_OBJECT:
		objattr = (struct mfs_obj_attr *)p;
                for (i=0;i<(attr->len-4)/sizeof(*objattr);i++) {
                        printf("%d/%d ",
			       ntohl(objattr->fsid),
                               ntohl(objattr->subobj));
			objattr++;
                }
                break;
        }
	printf("\n");
}

static void dumpobj(int fsid)
{
	void *buf;
	u32 size;
	if (mfs_fsid_type(fsid) != MFS_TYPE_OBJ) {
		fprintf(stderr,"%d is not an object\n", fsid);
		exit(1);
	}
	size = mfs_fsid_size(fsid);
	buf = malloc(size);
	mfs_fsid_pread(fsid, buf, 0, size);
	parse_object(fsid, buf, dump_callback);
	free(buf);
	printf("}\n");
}


static void usage(void)
{
	printf("\n"
"usage: mfs_dumpobj [options] <path|fsid>\n"
);
	exit(1);
}


 int main(int argc, char *argv[])
{
	int fsid;
	int c;

	while ((c = getopt(argc, argv, "")) != -1 ){
		switch (c) {
		default:
			usage();
		}
	}

	argc -= optind;
	argv += optind;

	if (argc < 1) {
		usage();
	}

	mfs_init();
	fsid = mfs_resolve(argv[0]);
	dumpobj(fsid);

	return 0;
}
